#include <oxstd.h>
#include "oxlatex_config.h"
#include "stringlist.ox"

class OxLatex {
	decl m_DocumentClass;
	decl m_Packages;
	decl m_Lines;
	decl m_Filename;
	decl m_Author;
	decl m_Title;
	decl m_ChangedSinceLastCompile;
	OxLatex();

	// preamble functions
	DocumentClass(const documentclass);
	UsePackage(const package);

	// generating functions
	Print(...);
	Println(...);
	Equation(...);
	NewLine();
	Title(const title);
	Author(const title);
	Section(const title);
	Subsection(const title);

	// output functions
	Compile();
	Preview();
	SetFilename(const filename);
	WriteToFile(const file);

	// internal functions
	PrintArray(const args);
}

OxLatex::OxLatex() {
	DocumentClass("article");
	m_Lines    = new StringList();
	m_Packages = new StringList();
	m_Filename = "oxlatex_temp";
	m_Author   = "";
	m_Title    = "";
	m_ChangedSinceLastCompile = TRUE;
}

OxLatex::SetFilename(const filename) {
	m_Filename = filename;
}

OxLatex::UsePackage(const package) {
	m_Packages.Add(package);
}

formatLatexMatrix(const A) {
	if ((rows(A) == 1) && (columns(A) == 1))
		return sprint(double(A));
	decl out = "";
	for (decl r = 0; r < rows(A); r++) {
		out += (r>0)?" \\\\ ":"";
		for (decl c = 0; c < columns(A); c++)
			out += ((c>0)?" & ":"") + sprint(A[r][c]);
	}
	return "\\ensuremath{\\left[\\begin{matrix}" + out + "\\end{matrix}\\right]}";
}

OxLatex::Equation(...) {
	m_Lines.Add("\\[");
	PrintArray(va_arglist());
	m_Lines.Add("\\]");
}

OxLatex::Println(...) {
	PrintArray(va_arglist());
	NewLine();
}

OxLatex::Print(...) {
	PrintArray(va_arglist());
}

OxLatex::NewLine() {
	m_Lines.Add("");
	m_ChangedSinceLastCompile = TRUE;
}

OxLatex::PrintArray(const args) {
	decl outstr = "";
	for (decl i = 0; i < sizeof(args); i++) {
		if (ismatrix(args[i]))
			outstr += formatLatexMatrix(args[i]);
		else
			outstr += sprint(args[i]);
	}
	m_Lines.Add(outstr);
	m_ChangedSinceLastCompile = TRUE;
}

OxLatex::DocumentClass(const documentclass) {
	m_DocumentClass = documentclass;
	m_ChangedSinceLastCompile = TRUE;
}

OxLatex::Author(const author) {
	m_Author = author;
	m_ChangedSinceLastCompile = TRUE;
}

OxLatex::Title(const title) {
	m_Title = title;
	m_ChangedSinceLastCompile = TRUE;
}

OxLatex::Section(const title) {
	Print("\\section{", title, "}");
}

OxLatex::Subsection(const title) {
	Print("\\subsection{", title, "}");
}

OxLatex::WriteToFile(const file) {

	// write preamble
	fprintln(file, "%% Warning: this file was generated by oxlatex -- do not edit");
	fprintln(file, "\\documentclass{", m_DocumentClass, "}\n");
	for (decl i = 0; i < m_Packages.Count(); i++) 
		fprintln(file, "\\usepackage{", m_Packages.Get(i), "}");
	fprintln(file, "\\newcommand\\oxLaTeX{{{\\tt ox}\LaTeX}}\n");

	// start document
	fprintln(file, "\\begin{document}");

	// write title if required
	if ((m_Title != "") && (m_Author != "")) {
		fprintln(file, "\\title{", m_Title, "}");
		fprintln(file, "\\author{", m_Author, "}");
		fprintln(file, "\\maketitle\n");
	}

	// write contents
	m_Lines.WriteToFile(file);

	// end document
	fprintln(file, "\\end{document}");
}

OxLatex::Compile() {
	decl file = fopen(sprint(m_Filename, ".tex"), "wt");
	WriteToFile(file);
	fclose(file);
	
	systemcall(sprint(LATEX, " ", m_Filename, ".tex"));
	m_ChangedSinceLastCompile = FALSE;
}

OxLatex::Preview() {
	if (m_ChangedSinceLastCompile)
		Compile();
	systemcall(sprint(PREVIEWER, " ", m_Filename, ".dvi"));
}
